<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Block;

class Popup extends \Magento\Framework\View\Element\Template {

    public $_helper;
    protected $scopeConfig;
    protected $_coreRegistry;
    protected $_storeManager;
    protected $_continent;
    protected $_countryContinent;
    protected $_request;

    public function __construct(
    \Magento\Framework\View\Element\Template\Context $context, \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper, \Magento\Framework\Registry $registry, \FiloBlu\ExtAmastyGeoipRedirect\Model\Continent $continent, \FiloBlu\ExtAmastyGeoipRedirect\Model\CountryContinent $countryContinent, array $data = []
    ) {

        $this->jsLayout = $data['jsLayout'];
        $this->_helper = $helper;
        $this->scopeConfig = $context->getScopeConfig();
        $this->_coreRegistry = $registry;
        $this->_storeManager = $context->getStoreManager();
        $this->_continent = $continent;
        $this->_countryContinent = $countryContinent;
        $this->_request = $context->getRequest();

        parent::__construct($context, $data);
    }

    public function getConfig() {
        return [
            "store_cookie_name" => $this->_helper->getStoreCookieName(),
            "redirect_allowed" => $this->_coreRegistry->registry('filoblu_geoip_redirect_allowed'),
            "popup_id" => "#popup-geoip",
            "data_continent_country" => $this->getAllCountries(),
            "country_link" => ".country-link, .current-country-link",
            "cookie_name" => $this->_helper->getCountryCookieName(),
            "cookie_duration" => 2592000,
            "shipping_to_link" => ".shipping-to-link",
            "language_list_container" => "#language-list .language-container",
            "language_link" => ".language-link",
            "current_choose_country_link" => ".current-choose-country-link",
            "continent_list_container" => "#continent-list",
            "is_homepage" => $this->checkIfHomePage()
        ];
    }

    private function checkIfHomePage() {
        $routeName = $this->_request->getRouteName();
        $action = $this->_request->getActionName();
        if ($routeName == 'cms' && $action == 'index') {
            return true;
        }
        else {
            return false;
        }
    }

    private function getAllCountries() {

        $result = array();
        $resultAllCountries = array();
        $resultAllContinentCountry = array();

        // Translated
        $resultAllCountriesTranslated = array();
        $resultAllContinentCountryTranslated = array();

        $allStores = $this->_storeManager->getStores();
        foreach ($allStores as $store) {

            $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', \Magento\Store\Model\ScopeInterface::SCOPE_STORES, $store->getId());
            $countries = explode(",", $countries);

            foreach ($countries as $country) {
                if ($country) {
                    $continent = $this->_countryContinent->getContinentByCountry($country);

                    $continentName = $continent->getContinent();
                    $countryName = $this->_helper->getCountryName($country, "en_US");

                    // Translated
                    $continentNameTranslated = _($continent->getContinent());
                    $countryNameTranslated = $this->_helper->getCountryName($country);


                    array_push($resultAllCountries, ["label" => $countryName, "country_id" => $country, "store_id" => $store->getId(), "url" => $store->getBaseUrl()]);

                    // Translated
                    array_push($resultAllCountriesTranslated, ["label" => $countryNameTranslated, "country_id" => $country, "store_id" => $store->getId(), "url" => $store->getBaseUrl()]);

                    $countryData = [
                        "country_id" => $country,
                        "store_id" => $store->getId(),
                        "url" => $store->getBaseUrl()
                    ];


                    if (!isset($resultAllContinentCountry[$continentName])) {
                        $resultAllContinentCountry[$continentName]["position"] = $continent->getPosition();
                    }
                    if (!isset($resultAllContinentCountry[$continentName]["countries"][$countryName])) {
                        $resultAllContinentCountry[$continentName]["countries"][$countryName] = $countryData;
                    } else {
                        array_push($resultAllContinentCountry[$continentName]["countries"][$countryName], $countryData);
                    }

                    // Translated
                    if (!isset($resultAllContinentCountryTranslated[$continentNameTranslated])) {
                        $resultAllContinentCountryTranslated[$continentNameTranslated]["position"] = $continent->getPosition();
                    }
                    if (!isset($resultAllContinentCountryTranslated[$continentNameTranslated]["countries"][$countryNameTranslated])) {
                        $resultAllContinentCountryTranslated[$continentNameTranslated]["countries"][$countryNameTranslated] = $countryData;
                    } else {
                        array_push($resultAllContinentCountryTranslated[$continentNameTranslated]["countries"][$countryNameTranslated], $countryData);
                    }
                }
            }
        }


        // Order by continent position
        $this->aasort($resultAllContinentCountry, "position");

        // Translated
        $this->aasort($resultAllContinentCountryTranslated, "position");

        // Order by country name
        foreach ($resultAllContinentCountry as $continent => $continentData) {
            ksort($continentData["countries"]);
            $resultAllContinentCountry[$continent]["countries"] = $continentData["countries"];
        }

        // Order by country name Translated
        foreach ($resultAllContinentCountryTranslated as $continent => $continentData) {
            ksort($continentData["countries"]);
            $resultAllContinentCountryTranslated[$continent]["countries"] = $continentData["countries"];
        }

        // Order Countries
        $this->aasort($resultAllCountries, "label");

        // Order Countries Translated
        $this->aasort($resultAllCountriesTranslated, "label");

        $result["continentcountry"] = $resultAllContinentCountry;
        $result["countries"] = $resultAllCountries;

        // Translated
        $result["continentcountrytranslated"] = $resultAllContinentCountryTranslated;
        $result["countriestranslated"] = $resultAllCountriesTranslated;

        return $result;
    }

    protected function aasort(&$array, $key) {
        $sorter = array();
        $ret = array();
        reset($array);
        foreach ($array as $ii => $va) {
            $sorter[$ii] = $va[$key];
        }
        asort($sorter);
        foreach ($sorter as $ii => $va) {
            $ret[$ii] = $array[$ii];
        }
        $array = $ret;
    }

    protected function _toHtml() {
        if (!$this->scopeConfig->getValue('amgeoipredirect/general/enable', \Magento\Store\Model\ScopeInterface::SCOPE_STORES) || !$this->_scopeConfig->getValue('amgeoipredirect/country_store/enable_store', \Magento\Store\Model\ScopeInterface::SCOPE_STORES)) {
            return "";
        } else {
            return parent::_toHtml();
        }
    }

}
